@@ -5,13 +5,15 @@
 package main
 
 import (
+	"bytes"
 	"flag"
-	"io"
 	"io/ioutil"
 	"log"
 	"net/http"
 	"os"
 	"os/exec"
+	"path/filepath"
+	"regexp"
 	"runtime"
 	"strconv"
 	"text/template"
@@ -25,25 +27,11 @@ var (
 var (
 	// a source of numbers, for naming temporary files
 	uniq = make(chan int)
-	// the architecture-identifying character of the tool chain, 5, 6, or 8
-	archChar string
 )
 
 func main() {
 	flag.Parse()
 
-	// set archChar
-	switch runtime.GOARCH {
-	case "arm":
-		archChar = "5"
-	case "amd64":
-		archChar = "6"
-	case "386":
-		archChar = "8"
-	default:
-		log.Fatalln("unrecognized GOARCH:", runtime.GOARCH)
-	}
-
 	// source of unique numbers
 	go func() {
 		for i := 0; ; i++ {
@@ -69,55 +57,15 @@ func FrontPage(w http.ResponseWriter, req *http.Request) {
 }
 
 // Compile is an HTTP handler that reads Go source code from the request,
-// compiles and links the code (returning any errors), runs the program, 
+// runs the program (returning any errors),
 // and sends the program's output as the HTTP response.
 func Compile(w http.ResponseWriter, req *http.Request) {
-	// x is the base name for .go, .6, executable files
-	x := os.TempDir() + "/compile" + strconv.Itoa(<-uniq)
-	src := x + ".go"
-	obj := x + "." + archChar
-	bin := x
-	if runtime.GOOS == "windows" {
-		bin += ".exe"
-	}
-
-	// write request Body to x.go
-	f, err := os.Create(src)
-	if err != nil {
-		error_(w, nil, err)
-		return
-	}
-	defer os.Remove(src)
-	defer f.Close()
-	_, err = io.Copy(f, req.Body)
-	if err != nil {
-		error_(w, nil, err)
-		return
-	}
-	f.Close()
-
-	// build x.go, creating x.6
-	out, err := run(archChar+"g", "-o", obj, src)
-	defer os.Remove(obj)
+	out, err := compile(req)
 	if err != nil {
 		error_(w, out, err)
 		return
 	}
 
-	// link x.6, creating x (the program binary)
-	out, err = run(archChar+"l", "-o", bin, obj)
-	defer os.Remove(bin)
-	if err != nil {
-		error_(w, out, err)
-		return
-	}
-
-	// run x
-	out, err = run(bin)
-	if err != nil {
-		error_(w, out, err)
-	}
-
 	// write the output of x as the http response
 	if *htmlOutput {
 		w.Write(out)
@@ -126,6 +74,60 @@ func Compile(w http.ResponseWriter, req *http.Request) {
 	}
 }
 
+var (
+	commentRe = regexp.MustCompile(`(?m)^#.*\n`)
+	tmpdir    string
+)
+
+func init() {
+	// find real temporary directory (for rewriting filename in output)
+	var err error
+	tmpdir, err = filepath.EvalSymlinks(os.TempDir())
+	if err != nil {
+		log.Fatal(err)
+	}
+}
+
+func compile(req *http.Request) (out []byte, err error) {
+	// x is the base name for .go, .6, executable files
+	x := filepath.Join(tmpdir, "compile"+strconv.Itoa(<-uniq))
+	src := x + ".go"
+	bin := x
+	if runtime.GOOS == "windows" {
+		bin += ".exe"
+	}
+
+	// rewrite filename in error output
+	defer func() {
+		if err != nil {
+			// drop messages from the go tool like '# _/compile0'
+			out = commentRe.ReplaceAll(out, nil)
+		}
+		out = bytes.Replace(out, []byte(src+":"), []byte("main.go:"), -1)
+	}()
+
+	// write body to x.go
+	body := new(bytes.Buffer)
+	if _, err = body.ReadFrom(req.Body); err != nil {
+		return
+	}
+	defer os.Remove(src)
+	if err = ioutil.WriteFile(src, body.Bytes(), 0666); err != nil {
+		return
+	}
+
+	// build x.go, creating x
+	dir, file := filepath.Split(src)
+	out, err = run(dir, "go", "build", "-o", bin, file)
+	defer os.Remove(bin)
+	if err != nil {
+		return
+	}
+
+	// run x
+	return run("", bin)
+}
+
 // error writes compile, link, or runtime errors to the HTTP connection.
 // The JavaScript interface uses the 404 status code to identify the error.
 func error_(w http.ResponseWriter, out []byte, err error) {
@@ -138,8 +140,14 @@ func error_(w http.ResponseWriter, out []byte, err error) {
 }
 
 // run executes the specified command and returns its output and an error.
-func run(cmd ...string) ([]byte, error) {
-	return exec.Command(cmd[0], cmd[1:]...).CombinedOutput()
+func run(dir string, args ...string) ([]byte, error) {
+	var buf bytes.Buffer
+	cmd := exec.Command(args[0], args[1:]...)
+	cmd.Dir = dir
+	cmd.Stdout = &buf
+	cmd.Stderr = cmd.Stdout
+	err := cmd.Run()
+	return buf.Bytes(), err
 }
 
 var frontPage = template.Must(template.New("frontPage").Parse(frontPageText)) // HTML template
