@@ -10,6 +10,12 @@ runtime·signame(int32)
 	return runtime·emptystring;
 }
 
+void
+runtime·sigenable(uint32 sig)
+{
+	USED(sig);
+}
+
 void
 runtime·resetcpuprofiler(int32 hz)
 {
